function PS311_vNSAint2(lambda, Tfinal)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function PS311_vNSAint2(lambda, Tfinal)
% ____________________________________________________________________________
% 18.311 (Principles of Applied Mathematics). MIT, Winter 2012. R. R. Rosales.
% ____________________________________________________________________________
%
% Demo to study how the numerical scheme (for the equation u_t + u_x = 0)
%
% (1-lambda)*u(n, k+1) + lambda*u(n+1, k+1) = u(n, k),  where lambda = dt/dx,
%
% behaves.  Here u(n, k) denotes the value of the solution at x = x_n = n*dx,
% and t = t_k = k*dt.    The implementation uses periodic boundary conditions
% in 0 < x < 2*pi and 0 < t < Tfinal,
% with a "chopped" cosine initial condition: u(x, 0) = max(0, - cos(x)).
%
% The default for Tfinal is 5.00 (if not given as an input).
%
% This scheme is the one analyzed in the  second example  of the introduction
% to the vNSA (von Neumann Stability Analysis) series of problems.   There it
% is shown that:
%                This scheme is stable for \lambda > lambda_c = 1.
%
% The purpose of this script is to illustrate this fact. What you will see is
% that
% --- If lambda < lambda_c, then: as dt, dx ---> 0 the solutions develop very
%     large amplitude grid scale oscilations (instability).
% --- If lambda > lambda_c, then: as dt, dx ---> 0 the solutions  converge to
%     the solutions of the pde [u of the form u = f(x-t)].
% --- What happens for lambda = lambda_c?
%
% - THE CODE HAS BEEN WRITTEN SO THAT REPLACING THE INITIAL CONDITIONS,
% - AND OTHER SUCH THINGS, IS EASY.
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Length = 2*pi;% ---------- Period in space of the solutions.        See below.
Npoint = 50:70:330;% ----- Array with numerical grid sizes.         """"""""""
Test   = -1;% ------------ This is used to turn on/off (Test = 1/-1) a test of
%                          the linear solver for the scheme implicit equations
%                          --- which are solved using FFT's.
Freeze =  250;% ---------- If the errors grow, to the point where the solution
%                          reaches the size specified by Freeze,   we stop any
%                          further growth by scaling down to size Freeze at
%                          each time step. This avoids problems with plotting:
%                          axis limits over-flows.
if nargin < 2;
   Tfinal = 5.00; end;% -- Final time for numerical integration.    See below.
% ============================================================================
% Parameters for the numerical integration.
%     Length = period in space for solution: 0 <= x <= Length.
%     Tfinal = time for integration:         0 <= t <= Tfinal.
%     Npoint = positive integer array with the number of points in the space
%              grid.   Integrations will be done with dx = Length/Npoint(n),
%              with n=1 ... length(Npoint).
%     lambda = value given to the numerical parameter in the scheme.
%
% ============================================================================
% Below strings containing descriptions of:
%   (1) Equation solved, (2) Scheme used, (3) Numerical parameter(s).
%   These are used for
%   (A) Command line messages to the user [these should be in plain text].
%   (B) Titles and labels in the figures  [these should be in LaTeX format].
%
% --- Command line strings.
%
EQUATN = 'u_t + u_x = 0';
SCHEME = '(1-dt/dx)*u(n, k+1) + (dt/dx)*u(n+1, k+1) = u(n, k)';
LAMBDA = 'dt/dx';
%
% --- LaTeX strings.
%
EQNtex = 'u_t + u_x = 0';
SCHtex = '(1-dt/dx) u_n^{k+1} + (dt/dx) u_{n+1}^{k+1} = u_n^k';
%
% ============================================================================
% Command line messages generated by demo, stating what it does.
%
fprintf( '\n Finite differences numerical scheme demo: PS311_vNSAint2.\n')
fprintf(['\n EQUATION: ', EQUATN, '.\n'])
fprintf(['\n SCHEME:   ', SCHEME, '.\n'])
fprintf( '\n CONSISTENCY AND STABILITY: ')
fprintf( '\n As dx, dt --> 0, is there a critical value LAMBDA_cr > 0')
fprintf(['\n such that, for LAMBDA = ', LAMBDA, ...
         ' > LAMBDA_cr, the numerical'])
fprintf( '\n solution converges to the true solution?')
fprintf( '\n If so, find LAMBDA_cr. \n')
fprintf( '\n WARNING. In the unstable case, once the errors grow too large,')
fprintf( '\n          further growth is prevented by: Scaling the solution,')
fprintf( '\n          at each time step, down to a maximum fixed size.')
fprintf(['\n          Currently this size is ', num2str(Freeze), '.\n \n'])
%
% ============================================================================
% Request user input lambda if not in the script call.
%
if nargin < 1
   lambda = input(['  Enter the value of lambda = ', LAMBDA, ' = ']);
   if isempty(lambda); lambda = 1; end
   fprintf('\n')
end
if lambda <= 0; lambda = 1; end
%
% ============================================================================
% Define initial conditions (stored in u0) and integrate.
% Note that changing the initial conditions is very simple.
%
for n = 1:length(Npoint)
   N  = Npoint(n);   dx = Length/N;   x = Length*(1:N)/N;   dt = lambda*dx;
   u0 = max(0, - cos(x)) + (rand(1, N) - 0.5)/10^3;
   U  = u0;% --- U will have the solution, one time level per row.
   T  =  0;% --- T will have the times computed.
   t  = dt;% --- t will have the time to be computed next.
   up = u0;% --- In each time step, up contains the solution at the current
   un = u0;%     time, and the solution at the next time goes into un.
   er =  0;% --- This is used by the linear solver testing step below.
   %
   while t <= Tfinal + dt
      %
      % --- Here the scheme is implemented. The equation determining un is
      %     (1-lambda)*un + lambda*S*un = u0;                 [SCHEME]
      %     where S is the shift operator (S*v)(j) = v(j+1);
      %           [from periodicity v(length(v)+1) = v(1);]
      %
      %     [SCHEME] is easily solved using FFT's.
      %
      a  = 2*pi/N;
      G  = 1./(1-lambda+lambda*exp(i*a*(0:N-1)));
      un = real(ifft(G.*fft(up)));
      %
      % --- TEST the linear solver.
      if Test > 0; Su = [un(2:N), un(1)]; u1 = (1-lambda)*un + lambda*Su;
                   er = max(max(abs(up-u1)), er); end
      % --- TEST ENDS.
      %
      % --- FREEZE ERROR GROWTH ONCE ERRORS GET LARGE ENOUGH.
      %     We do this by re-scaling solution to maximum allowed size.
      %
      uM = max(abs(un));
      if uM > Freeze; un = (Freeze/uM)*un; end
      %
      % --- Store updated solution at new time.
      %
      U = [U; un];   T = [T; t];   up = un;   t = t + dt;
   end
   if Test > 0; fprintf([' Error in solving linear equations <= ', ...
                         num2str(er), '\n']); end
   zmax = max(max(U));
   zmin = min(min(U));
   MakeTheSchemeFigure(n, N)
   whitebg('k')
   mesh([0, x], T, [U(:, N), U]);
   hold on
   grid on
   rotate3d on
   axis([0 Length 0 (t-dt) zmin zmax])
   set(gca, 'LineWidth',   2, ...
            'Position',    [0.1500 0.1500 0.7350 0.7050], ...
            'FontWeight', 'bold', ...
            'FontSize',    16)
   colormap(hsv)
   %
   % --- Titles, labels, and other displayed info.
   %
   text('String',               LAMBDA, ...
        'Units',               'Normalized', ...
        'Position',             [0.780 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               [' = ', num2str(lambda, '%5.3f')], ...
        'Units',               'Normalized', ...
        'Position',             [0.930 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              'dx', ...
        'Units',               'Normalized', ...
        'Position',             [0.780 1.00], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              ' = 2 \pi / N', ...
        'Units',               'Normalized', ...
        'Position',             [0.930 1.00], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              'N', ...
        'Units',               'Normalized', ...
        'Position',             [0.780 0.93], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               [' = ', num2str(N)], ...
        'Units',               'Normalized', ...
        'Position',             [0.930 0.93], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               SCHtex, ...
        'Units',               'Normalized', ...
        'Position',             [-0.10 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               EQNtex, ...
        'Units',               'Normalized', ...
        'Position',             [-0.10 0.02], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   xlabel('{\bf x}',  'FontSize', 22)
   ylabel('{\bf t}',  'FontSize', 22)
   zlabel('{\bf u}',  'FontSize',             22, ...
                      'HorizontalAlignment', 'Right', ...
                      'Rotation', 0)
end
if Test > 0; fprintf('\n'); end
fprintf(' Rotate 3D is on in the figures.\n')
fprintf(' Type "close all" to close the figures. \n \n')
% ============================================================================

function MakeTheSchemeFigure(n, N)
% ============================================================================
SSz = get(0, 'ScreenSize');
SW  = SSz(3);% ------------ Screen Width.
SH  = SSz(4);% ------------ Screen Height.
Sml = min(SW, SH);% ------- Screen minimum length.
% ------------------------- Lengths below in fractions of Sml.
dX = 0.03*n;% --- Figure window distance from screen right edge.
dY = 0.03*n;% --- Figure window distance from screen top   edge.
fW = 6.55*0.07;% --- Figure window width.
fH = 5.00*0.07;% --- Figure window height.
% ------------------------- Figure position.
POF = [SW - (fW + dX)*Sml, SH - (fH + dY)*Sml, fW*Sml, fH*Sml];
%
NOF = ['Solution with N = ', num2str(N), ' points in space.'];
%
figure('Name', NOF, 'NumberTitle', 'off', 'Position', POF)
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF